"""

    Termux String Generator

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

class StringGenerator():
    @staticmethod
    def generateString(character: str,lenght: int) -> str:
        output = ""
        for i in range(lenght):
            output += character
        return output



"""

    Termux Validation Exception

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

class ValidationException(Exception):
    def __init__(self, *args: object) -> None:
        super().__init__(*args)


"""

    Termux Sequential Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from termuxmenu import ValidationException, Menu, ErrorLog, Question

class SequentialMenu(Menu):
    def __init__(self,questions: list,escape: str,error_log: ErrorLog,validation: list = None) -> None:
        self.questions = questions
        self.escape = escape
        self.error_log = error_log
        self.validation = validation
        self.output = list()
        self.step = 0
    def incrementStep(self):
        self.step += 1
    def decrementStep(self):
        self.step -= 1
    def resetStep(self):
        self.step = 0
    def loop(self):
        while self.output.count() < self.questions.count():
            try:
                if self.validation != None:
                    self.output.append(Question.ask(self.questions[self.step],self.escape,self.validation[self.step]))
                else:
                    self.output.append(Question.ask(self.questions[self.step],self.escape))
            except ValidationException as validationException:
                self.error_log.add(validationException)
                self.decrementStep()
            except Exception as ioException:
                self.error_log.add(ioException)
            finally:
                self.incrementStep()
    def getOutput(self) -> list:
        return self.output
    def update(self):
        self.loop()
    def show(self):
        self.update()


"""

    Termux Error

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from abc import abstractclassmethod

class Error():
    @abstractclassmethod
    def clear(self):
        pass
    @abstractclassmethod
    def size(self) -> int:
        pass
    @abstractclassmethod
    def get(self) -> list:
        pass
    @abstractclassmethod
    def add(self,string: str):
        pass


"""

    Termux Console

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from abc import abstractclassmethod

class Console():
    @abstractclassmethod
    def start(self):
        pass


"""

    Termux Question

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

import re
from termuxmenu import ValidationException

class Question():
    shell = "> "
    @staticmethod
    def ask(message: str,escape: str) -> str:
        print(("\n" + escape + "{}").format(message),end="")
        return input(("\n" + escape + "{}").format(Question.shell))
    @staticmethod
    def ask(message: str,escape: str,pattern: str) -> str:
        print(("\n" + escape + "{}").format(message),end="")
        answer = input(("\n" + escape + "{}").format(Question.shell))
        if not re.match(pattern=pattern,string=answer):
            raise ValidationException("Validation failed, doesn't match with: {}".format(pattern))
        return answer



"""

    Termux Command Parser

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from abc import abstractclassmethod

class CommandParser():

    @abstractclassmethod
    def parseBlock(self,command: str) -> int:
        pass

    @staticmethod
    def parseCommand(command: str,parser) -> int:
        return CommandParser.parseBlock(parser,command)


"""

    Termux Default Error

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from termuxmenu import Error

class DefaultError(Error):
    def __init__(self) -> None:
        super().__init__()
        self.errors = list()
    def get(self) -> list:
        return self.errors
    def clear(self):
        self.errors.clear()
    def size(self) -> int:
        return self.errors.count()
    def add(self, string: str):
        return self.errors.append(string)


"""

    Termux Options Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

from termuxmenu import DefaultMenu

class OptionsMenu(DefaultMenu):
    def __init__(self, escape: str,options: list) -> None:
        super().__init__(escape)
        self.options = options


"""

    Termux Option Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

from termuxmenu import OptionsMenu, Encapsulate, StringGenerator

class OptionMenu(OptionsMenu):

    TITLE_LENGHT = 35
    SPACING = 2

    def __init__(self, escape: str, options: list,title: str,format: str,number: int = 1,enable_numbers: bool = True) -> None:
        super().__init__(escape, options)
        self.title = title
        self.format = format
        self.number = number
        self.enable_numbers = enable_numbers
    def generateTitle(self) -> str:
        return Encapsulate.inlineEncapsulate(self.title,OptionMenu.TITLE_LENGHT,OptionMenu.SPACING)
    def generateBottom(self) -> str:
        return StringGenerator.generateString('-',OptionMenu.TITLE_LENGHT)
    def showOptions(self):
        n = self.number
        for option in self.options:
            if self.enable_numbers:
                print(("\n" + self.escapeCharacters + "%-5d" + self.format) % (n,option),end="")
            else:
                print(("\n" + self.escapeCharacters + self.format) % option,end="")
            n += 1
    @staticmethod
    def setTitleLenght(lenght: int):
        OptionMenu.TITLE_LENGHT = lenght
    @staticmethod
    def setSpacing(spacing: int):
        OptionMenu.SPACING = spacing
    def update(self):
        print("\n" + self.escapeCharacters + "%s" % self.generateTitle(),end="")
        self.showOptions()
        print("\n" + self.escapeCharacters + "%s\n" % self.generateBottom(),end="")


"""

    Termux Inline Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

from termuxmenu import OptionsMenu

class InlineMenu(OptionsMenu):
    MAX_ITEMS_PER_ROW = 5
    def __init__(self, escape: str, options: list,number: int) -> None:
        super().__init__(escape, options)
        self.number = number
    def showOptions(self) -> str:
        n = self.number
        c = 1
        output = ""
        for option in self.options:
            if c > InlineMenu.MAX_ITEMS_PER_ROW:
                output += "\n"
                c = 1
            if c == 1:
                output += self.escapeCharacters
            output += n + " " + option + " "
            c += 1
            n += 1
        return output
    def update(self):
        print("\n%s\n" % self.showOptions(),end="")


"""

    Termux Default Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

from abc import abstractclassmethod
from termuxmenu import Menu

class DefaultMenu(Menu):
    def __init__(self,escape: str) -> None:
        super().__init__()
        self.escapeCharacters = escape
    @abstractclassmethod
    def update(self):
        pass
    def show(self):
        self.update()


"""

    Termux Selection Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from abc import abstractclassmethod
from termuxmenu import DefaultMenu, Encapsulate, StringGenerator
import math

class SelectionMenu(DefaultMenu):
    MAX_ITEM_IN_PAGE = 5
    BOX_LENGHT = 35
    HEADER_SPACING = 2
    def __init__(self, escape: str,items: list,header: str) -> None:
        super().__init__(escape)
        self.items = items
        self.header = header
        self.max_page = math.ceil(self.items.count() / SelectionMenu.MAX_ITEM_IN_PAGE)
        self.selectionIndex = 0
        self.pageIndex = 1
        self.currentMinSelection = 0
    def generateTitle(self,title: str) -> str:
        return Encapsulate.inlineEncapsulate(title,SelectionMenu.BOX_LENGHT,SelectionMenu.HEADER_SPACING)
    def generateBottom(self) -> str:
        return StringGenerator.generateString('-',SelectionMenu.BOX_LENGHT)
    def previousPage(self):
        if self.currentMinSelection - SelectionMenu.MAX_ITEM_IN_PAGE < 0:
            raise Exception("MIN PAGE")
        self.currentMinSelection -= SelectionMenu.MAX_ITEM_IN_PAGE
        self.selectionIndex = self.currentMinSelection
        self.pageIndex -= 1
    def nextPage(self):
        if self.currentMinSelection + SelectionMenu.MAX_ITEM_IN_PAGE > (self.items.count() - 1):
            raise Exception("MAX PAGE")
        self.currentMinSelection += SelectionMenu.MAX_ITEM_IN_PAGE
        self.selectionIndex = self.currentMinSelection
        self.pageIndex += 1
    def previousItem(self):
        if self.selectionIndex - 1 < self.currentMinSelection:
            raise Exception("MIN ITEM")
        self.selectionIndex -= 1
    def nextItem(self):
        o = None
        try:
            o = self.items[self.selectionIndex + 1]
        except IndexError:
            pass
        if self.selectionIndex + 1 > (self.currentMinSelection + SelectionMenu.MAX_ITEM_IN_PAGE - 1):
            raise Exception("MAX ITEM")
        if o != None:
            self.selectionIndex += 1
    def select(self) -> object:
        return self.items[self.selectionIndex]
    @abstractclassmethod
    def showItem(self,o: object,selected: bool):
        pass
    def showPage(self):
        print("\n" + self.escapeCharacters + self.generateTitle("%s %d" % ("Page", self.pageIndex)),end="")
        print(self.header,end="")
        i = self.currentMinSelection
        while i < (self.currentMinSelection + SelectionMenu.MAX_ITEM_IN_PAGE):
            o = None
            try:
                o = self.items[i]
            except IndexError:
                pass
            if o != None:
                self.showItem(o,i == self.selectionIndex)
            i += 1
        print("\n%s\n" % (self.escapeCharacters + self.generateBottom(
            "%s %d/%d" % ("Page",self.pageIndex,self.max_page)
        )),end="")
    def update(self):
        self.showPage()
    def show(self):
        self.update()
    @staticmethod
    def setMaxItemInPage(max: int):
        SelectionMenu.MAX_ITEM_IN_PAGE = max


"""

    Termux Selection Menu

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from termuxmenu import DefaultInteractiveMenu, SelectionMenu, DefaultMenu, CommandParser, ErrorLog

class SelectionInteractiveMenu(DefaultInteractiveMenu):
    def __init__(self, error_log: ErrorLog, option_menu: DefaultMenu, command_parser: CommandParser, shell: str,selection_menu: SelectionMenu) -> None:
        super().__init__(error_log, option_menu, command_parser, shell)
        self.selection_menu = selection_menu
    def loopBlock(self):
        self.selection_menu.show()
        self.option_menu.show()


"""

    Termux Menu Interface

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

from abc import abstractclassmethod

class Menu():
    @abstractclassmethod
    def show(self):
        pass


"""

    Termux Error Log

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from termuxmenu import Menu, DefaultMenu, ErrorLog, CommandParser
from abc import abstractclassmethod

class DefaultInteractiveMenu(Menu):
    def __init__(self,error_log: ErrorLog,option_menu: DefaultMenu,command_parser: CommandParser,shell: str) -> None:
        super().__init__()
        self.error_log = error_log
        self.option_menu = option_menu
        self.command_parser = command_parser
        self.shell = shell
        self.exit = False
    @abstractclassmethod
    def outsideLoop(self):
        pass
    @abstractclassmethod
    def loopBlock(self):
        pass
    def loop(self):
        self.outsideLoop()
        while not self.exit:
            self.loopBlock()
            try:
                print(self.shell,end="")
                if CommandParser.parseCommand(input(),self.command_parser) == -1:
                    self.exit = True
            except Exception as error:
                self.error_log.add(error)
    def update(self):
        self.loop()
    def show(self):
        self.update()


"""

    Termux Error Log

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from termuxmenu import DefaultError, OptionMenu

class ErrorLog(DefaultError):
    def show(self,escape: str,title: str):
        self.menu = OptionMenu(options=self.get(),escape=escape,title=title,format="%s")
        self.menu.show()


"""

    Termux Encapsulate

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/20 

"""

import math
from termuxmenu import StringGenerator

class Encapsulate():
    @staticmethod
    def encapsulateString(message: str,escape: str,space: str = None):
        if space == None:
            Encapsulate.encapsulate(message=message,character='-',escape=escape,lenght=len(message))
        else:
            Encapsulate.encapsulate(message=message,character=space,escape=escape,lenght=len(message))
    @staticmethod
    def inlineEncapsulate(message: str,lenght: int,spacing: int) -> str:
        firstStage = math.ceil(((lenght - len(message)) / 2) - math.ceil(spacing / 2))
        secondStage = math.floor(((lenght - len(message)) / 2) - math.floor(spacing / 2))
        firstSpacing = math.ceil(spacing / 2)
        secondSpacing = math.floor(spacing / 2)
        return (
            StringGenerator.generateString(character="-",lenght=firstStage) 
            + StringGenerator.generateString(character=" ",lenght=firstSpacing)
            + message
            + StringGenerator.generateString(character=" ",lenght=secondSpacing)
            + StringGenerator.generateString(character="-",lenght=secondStage)
        )
    @staticmethod
    def inlineCenter(message: str,character: str,lenght: int) -> str:
        return '{:{}^{}}'.format(message,character,lenght)
    @staticmethod
    def encapsulate(message: str,character: str,escape: str,lenght: int):
        firstPhaseCheck = False
        secondPhaseCHeck = False
        stringCheck = False
        for x in range((lenght + 1) * 3):
            if not firstPhaseCheck:
                if x == 0:
                    print("\n" + escape,end="")
                print(character, end="")
                if x == (lenght + 1):
                    firstPhaseCheck = True
            elif not secondPhaseCHeck:
                if x == (lenght + 2):
                    print("\n" + escape + "|", end="")
                elif x == ((lenght * 2) + 1):
                    print("|", end="")
                    secondPhaseCHeck = True
                elif not stringCheck:
                    print(message,end="")
                    stringCheck = True
            else:
                if x == ((lenght * 2) + 2):
                    print("\n" + escape + character,end="")
                print(character,end="")


"""

    Termux Default Console

    AUTHOR: Carlos Pomares (https://www.github.com/pomaretta) 
    LAST REV: 2021/04/22

"""

from abc import abstractclassmethod
from termuxmenu import Console, ErrorLog

class DefaultConsole(Console):
    def __init__(self,error_log: ErrorLog) -> None:
        super().__init__()
        self.error_log = error_log
    @abstractclassmethod
    def main(self):
        pass
    def start(self):
        self.main()


from termuxmenu.console import Console
from termuxmenu.default_console import DefaultConsole
from termuxmenu.default_error import DefaultError
from termuxmenu.default_interactive_menu import DefaultInteractiveMenu
from termuxmenu.default_menu import DefaultMenu
from termuxmenu.encapsulate import Encapsulate
from termuxmenu.error import Error
from termuxmenu.inline_menu import InlineMenu
from termuxmenu.logger import ErrorLog
from termuxmenu.menu import Menu
from termuxmenu.option_menu import OptionMenu
from termuxmenu.options_menu import OptionsMenu
from termuxmenu.parser import CommandParser
from termuxmenu.question import Question
from termuxmenu.selection_interactive_menu import SelectionInteractiveMenu
from termuxmenu.selection_menu import SelectionMenu
from termuxmenu.sequential_menu import SequentialMenu
from termuxmenu.string_generator import StringGenerator
from termuxmenu.validation_exception import ValidationException